/************************************************
AVR Code to send PPM servo signals to 4017 chip

-- main.c --

Created: 9/30/09
Last Modified: 10/7/09
************************************************/

#include <avr/io.h>
#include <stdlib.h>
//#include <avr/signal.h>
#include <avr/interrupt.h>

#include <ctype.h>
#include <stdint.h>
#include <stdio.h>
#include <inttypes.h>

#include "sys_time_hw.h"
#include "usart.h"
#include "servos_4017.h"

#define ON	1
#define OFF 0

#define Green_LED	3
#define Red_LED		4

void init_chip(void);
void turn_Big_LED(unsigned char On_Off);
void turn_LED(unsigned char LED_Num, unsigned char On_Off);
void check_buttons(void);

/* global variable definitions */
/* volatile means value can change without explicit assignment */

//extern volatile uint8_t servo;

uint16_t cpu_time_sec;

uint16_t sWidth[10], sInc[10];

// --------------------------------------------------------------------------------------------------------------------------------------------
// Big LED on Olimex board = PortC:5

void turn_Big_LED(unsigned char On_Off)
{
	if (On_Off == ON) {
		PORTC &= ~(1 << 5); // clear pin turns LED on
		}
	else
		{
		PORTC |= (1 << 5);	// set pin turns LED off
		}
}

// --------------------------------------------------------------------------------------------------------------------------------------------
// The Red/Green LEDs are on PortD:3&4
// and are wired in backwards compared to Big LED

void turn_LED(unsigned char LED_Num, unsigned char On_Off)
{
	if (On_Off == OFF) {
		PORTC |= (1 << LED_Num);	// set pin turns LED on
		}
	else
		{
		PORTC &= ~(1 << LED_Num); // clear pin turns LED off
		}
}

// --------------------------------------------------------------------------------------------------------------------------------------------

void toggle_LED(unsigned char LED_Num)
{
	PORTC ^= (1 << LED_Num);
}

// --------------------------------------------------------------------------------------------------------------------------------------------

void check_buttons(void)
{
	if (bit_is_clear(PIND, 2)) {
		turn_LED( 0, ON );
	//	printf("Button 0 : %d\r\n", (int)servo);
		loop_until_bit_is_set(PIND, 2);
		turn_LED( 0, OFF );
		}

	if (bit_is_clear(PIND, 3)) {
		turn_LED( 1, ON );
		printf("Button 1\r\n");
		loop_until_bit_is_set(PIND, 3);
		turn_LED( 1, OFF );
		}

	if (bit_is_clear(PIND, 4)) {
		turn_LED( 2, ON );
		printf("Button 2\r\n");
		loop_until_bit_is_set(PIND, 4);
		turn_LED( 2, OFF );
		}

	if (bit_is_clear(PIND, 5)) {
		turn_LED( Green_LED, ON );
		turn_LED( Red_LED, OFF );
		
		printf("Button 3\r\n");
		
		loop_until_bit_is_set(PIND, 5);
		turn_LED( Red_LED, ON );
		turn_LED( Green_LED, OFF );
		
		/*
		printf("Secs: %d\r\n", cpu_time_sec);
		printf("Tics: %d\r\n", cpu_time_ticks);
		printf("TPS: %d\r\n", TICKS_PER_SEC);
		*/
		
		}
}

// -----------------------------------------------------------------------------------------------------

void init_chip(void)
{
  DDRB = 0xFE;			// Initialize port directions
  DDRC = 0xFF;			// a '1' bit indicates a output pin
  DDRD = 0xC2;			// D0 is Rx, D1 is Tx  0b 1100 0010

  PORTC = 0xFF;			// LEDs

  Init_USART();
  fdevopen(Uart_putchar, NULL, 0);
  
  sys_time_init();		/* Initialize timers */
  actuators_init();		/* PWM pulse widths to 4017 decade counter */
  
  sei();				/* Enable interrupts */

  toggle_LED(3);
}

// -----------------------------------------------------------------------------------------------------

int main (void)
{
  int n, last_time=0;
  
  init_chip();		/* Initialize port directions */

  printf("\r\nHello World.\r\n");
  printf("  Servo Width: %d\r\n", servo_widths[1]);

  for (n=1; n<9; n++) 
	{
	sWidth[n] = 900;
    sInc[n] = n * 3;
	}
  
// ----------------------------------------------------
	
  /* do an infinite loop */

  for (;;) {

	if (sys_time_periodic())	// this goes off 18 times per second
		{
		//printf("Sys Time Periodic.\r\n");
		toggle_LED(3);
		
		cpu_time_ticks += 512;
		if (cpu_time_ticks > TICKS_PER_SEC) {
			cpu_time_ticks -= TICKS_PER_SEC;
			cpu_time_sec++;
			}
		
		for (n=1; n<9; n++) 
			{
			sWidth[n] += sInc[n];
			SetServo(n, sWidth[n]);
			if ((sWidth[n] > 2100) || (sWidth[n] < 900))
				sInc[n] = -sInc[n];
			}
			
//		sWidth += sInc;
//		SetServo(2, sWidth);
//		if ((sWidth > 2100) || (sWidth < 900))
//			sInc = -sInc;
		}
	
	check_buttons();
	
	if (last_time != cpu_time_sec)		// this goes off once per second
		{
		last_time = cpu_time_sec;
		toggle_LED(5);
					
		//printf("Looping...  ");
		
		// tmp_radio = ppm_pulses[RADIO_%s] - SYS_TICS_OF_USEC(%d);
		}
  }

return 0;
}

// -----------------------------------------------------------------------------------------------------
// End of File

