// File: sys_time_hw.c

#include <inttypes.h>
#include <avr/signal.h>
#include <avr/interrupt.h>

//#include CONFIG
#include "std.h"
#include "sys_time_hw.h"

uint16_t cpu_time_ticks;

#if CLOCK == 8
volatile uint8_t tmr2_ov_cnt;
volatile bool_t tmr2_overflow;

SIGNAL(SIG_OVERFLOW2) {
  tmr2_ov_cnt++;
  tmr2_overflow = TRUE;
//  return;
}

#endif

/*
 * Enable Timer1 (16-bit) running at Clk/1 for the Global System Clock.
 *
 * Low frequency periodic tasks will be signaled by timer 2
 * running at Clk/1024.  For 16 Mhz clock, this will be every
 * 16384 microseconds, or 61.03515625 Hz.
 */

void sys_time_init( void ) 
{

  /* Timer1 @ Clk/1: System clock */
  TCCR1A = 0x00;
  TCCR1B = _BV(CS10);


  /* Timer2 @ Clk/1024: Periodic clock */
#if defined (__AVR_ATmega8__)
  TCCR2         = _BV(CS20) | _BV(CS21) | _BV(CS22);
  TIMSK |= (1<<TOIE2);
  
#elif defined (__AVR_ATmega128__)
#if CLOCK == 16
  TCCR2         = _BV(CS20) | _BV(CS22);
#elif CLOCK == 8
  TCCR2         = _BV(CS22);
//  sbi( TIMSK, TOIE2 );
  TIMSK |= (1<<TOIE2);
  
#else
#error "Unknown CLOCK"
#endif
#else
#warning "Unknown arch"
#endif

  cpu_time_sec = 0;
  cpu_time_ticks = 0;
}

