// usart.c

#include <avr/io.h>
#include <stdlib.h>
#include <avr/interrupt.h>

#include <ctype.h>
#include <stdint.h>
#include <stdio.h>
#include <inttypes.h>

#include "usart.h"

#define BAUD 19200

// ------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

#if defined (__AVR_ATmega8__)
int Uart_putchar(char character)
{
	// Send characters to the serial line
	
	loop_until_bit_is_set(UCSRA, UDRE); // Wait until the last character is sent
	UDR = character;
	return 0;
}
#else
int Uart_putchar(char character, FILE *stream)
{
	// Send characters to the serial line
	
	loop_until_bit_is_set(UCSR0A, UDRE0); // Wait until the last character is sent
	UDR0 = character;
	return 0;
}
#endif

// ------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
// USART Mode: Asynchronous

#if defined (__AVR_ATmega8__)
void Init_USART(void)
{	
	UBRRH = 0x00; // write high byte first
	UBRRL = ((F_CPU/16)/BAUD)-1; // set baud rate
	UCSRA = 0x00;
	UCSRB |= (1 << 3); // TXEN ON, TX ON,        xxxx 1xxx
//	UCSRB |= (1 << 4); // RXEN, RX ON,           xxx1 1xxx
//	UCSRB |= (1 << 7); // RXIE, RX interrupt ON, 1xx1 1xxx
	UCSRC = 0x86; // 8 Data, 1 Stop, No Parity   1xxx x11x
}
#else
void Init_USART(void)
{	
	UBRR0H = 0x00; // write high byte first
//	UBRR0L = ((F_CPU/16)/BAUD)-1;	// set baud rate
	UBRR0L = 8;	// set baud rate = 57600 - NOTE: formula produced the value 7 which didn't work.
	UCSR0A = 0x00;
	UCSR0B |= (1 << 3); // TXEN ON, TX ON,        xxxx 1xxx
//	UCSR0B |= (1 << 4); // RXEN, RX ON,           xxx1 1xxx
//	UCSR0B |= (1 << 7); // RXIE, RX interrupt ON, 1xx1 1xxx
	UCSR0C = 0x86; // 8 Data, 1 Stop, No Parity   1xxx x11x
}
#endif

// ------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
// End of File
